#pragma once

#include "page_mapper.h"
#include "pages.h"
#include "pipe_accessor.h"

/* Pipe paging support, maintaining the sequence of active regions or sections
   in a pipe. */

class PipePaging
{
protected:
    Memory *_memory;
    Pages *_pages;
    PageQueue *_queue;

    /* Regions acting as files with their own accessors. */

    PageMapper *_regions[2];
    PipeAccessor _accessors[2];

    /* The first region is initially exposed to both reader and writer. */

    int _reading = 0, _writing = 0;

    /* Pipe section/region size. */

    offset_t _size;

    /* Endpoint status. */

    unsigned int _endpoints = 2;

public:
    explicit PipePaging(Memory *memory, offset_t size);

    virtual void detach();

    virtual offset_t region_size()
    { return _size; }

    /* Region management. */

    virtual PageMapper *add_region();

    virtual PageMapper *current_region();

    virtual PageMapper *next_region();
};

// vim: tabstop=4 expandtab shiftwidth=4
